@@ -10,6 +10,9 @@ read_dicom <- function(fname, verbose, extra) {
   
   sop_class_uid <- rawToChar(res$sop_class_uid)
   manuf         <- rawToChar(res$manuf)
+      
+  if (verbose) cat(paste("SOP class UID :", sop_class_uid), "\n")
+  if (verbose) cat(paste("Manufacturer  :", manuf), "\n")
   
   if (grepl("SIEMENS", manuf, ignore.case = TRUE)) {
     if (sop_class_uid == "1.3.12.2.1107.5.9.1") {
@@ -26,14 +29,17 @@ read_dicom <- function(fname, verbose, extra) {
     }
   } else if (grepl("Philips", manuf)) {
     if (sop_class_uid == "1.3.46.670589.11.0.0.12.1") {
-      return(read_philips_priv_dicom(fraw, extra))
+      if (verbose) cat("Philips private DICOM MRS found.\n")
+      return(read_philips_priv_dicom(fraw, extra, verbose))
     } else if (sop_class_uid == "1.2.840.10008.5.1.4.1.1.4.2") {
-      return(read_philips_dicom(fraw, extra))
+      if (verbose) cat("Philips DICOM MRS found.\n")
+      return(read_philips_dicom(fraw, extra, verbose))
     } else {
       stop(paste0("Unsupported SOP class UID : ", sop_class_uid,
                   ". This doesn't look like MRS data."))
     }
   } else if (grepl("UIH", manuf)) {
+      if (verbose) cat("UIH DICOM MRS found.\n")
       return(read_uih_dicom(fraw, extra, verbose))
   } else {
     stop(paste0("DICOM MRS manufacturer format: \"", trimws(manuf),
@@ -130,18 +136,20 @@ read_uih_dicom <- function(fraw, extra, verbose) {
 read_siemens_dicom <- function(fraw, extra, verbose) {
   
   # list of tags to pull from the dicom file
-  tags <- list(data    = "5600,0020",
-               fs      = "0018,9052",
-               ft      = "0018,9098",
-               ipp     = "0020,0032",
-               iop     = "0020,0037",
-               pixsp   = "0028,0030",
-               slice_t = "0018,0050",
-               rows    = "0028,0010",
-               cols    = "0028,0011",
-               slices  = "0018,9159",
-               Npt     = "0028,9002",
-               te      = "0018,9082")
+  tags <- list(data        = "5600,0020",
+               fs          = "0018,9052",
+               ft          = "0018,9098",
+               ipp         = "0020,0032",
+               iop         = "0020,0037",
+               pixsp       = "0028,0030",
+               slice_t     = "0018,0050",
+               rows        = "0028,0010",
+               cols        = "0028,0011",
+               slices      = "0018,9159",
+               Npt         = "0028,9002",
+               te          = "0018,9082",
+               series_desc = "0008,103E",
+               series_num  = "0020,0011")
 
   # read em out
   dcm_res <- dicom_reader(fraw, tags)
@@ -198,10 +206,13 @@ read_siemens_dicom <- function(fraw, extra, verbose) {
                   c(col_ori * res[3], 0),
                   c(sli_vec * res[4], 0),
                   c(pos_vec_affine, 1))
+  
   affine[1:2,] <- -affine[1:2,]
   
-  meta <- list(EchoTime = te,
-               Manufacturer = "Siemens")
+  meta <- list(EchoTime          = te,
+               Manufacturer      = "Siemens",
+               SeriesDescription = rawToChar(dcm_res$series_desc),
+               SeriesNumber      = as.integer(rawToChar(dcm_res$series_num)))
   
   mrs_data <- mrs_data(data = data, ft = ft, resolution = res, ref = ref,
                        nuc = nuc, freq_domain = freq_domain, affine = affine,
@@ -210,7 +221,7 @@ read_siemens_dicom <- function(fraw, extra, verbose) {
   return(mrs_data)
 }
 
-read_philips_dicom <- function(fraw, extra) {
+read_philips_dicom <- function(fraw, extra, verbose) {
   
   # list of tags to pull from the dicom file
   tags <- list(data    = "5600,0020",
@@ -292,19 +303,25 @@ read_philips_dicom <- function(fraw, extra) {
 }
   
 
-read_philips_priv_dicom <- function(fraw, extra) {
+read_philips_priv_dicom <- function(fraw, extra, verbose) {
   
   # list of tags to pull from the dicom file
-  tags <- list(data    = "2005,1270",
-               fs      = "2005,1030",
-               ft      = "2001,1083",
-               te      = "2005,1310")
+  tags <- list(data      = "5600,0020",
+               fs        = "2005,1030",
+               ft        = "2001,1083",
+               te        = "2005,1310",
+               Npts      = "0018,9127",
+               Nframes   = "0028,0008",
+               prot_name = "0018,1030")
   
   dcm_res  <- dicom_reader(fraw, tags)
   
-  fs  <- readBin(dcm_res$fs, "double", size = 4, n = 2)[1]
-  ft  <- as.numeric(rawToChar(dcm_res$ft)) * 1e6
-  te  <- readBin(dcm_res$te, "double", size = 4) / 1e3
+  fs        <- readBin(dcm_res$fs, "double", size = 4, n = 2)[1]
+  ft        <- as.numeric(rawToChar(dcm_res$ft)) * 1e6
+  te        <- readBin(dcm_res$te, "double", size = 4) / 1e3
+  Npts      <- readBin(dcm_res$Npts, "integer")
+  Nframes   <- as.numeric(rawToChar(dcm_res$Nframes))
+  prot_name <- rawToChar(dcm_res$prot_name)
   
   raw_vec <- readBin(dcm_res$data, "double", length(dcm_res$data) / 4, size = 4)
   data    <- raw_vec[c(TRUE, FALSE)] - 1i * raw_vec[c(FALSE, TRUE)]
@@ -329,5 +346,24 @@ read_philips_priv_dicom <- function(fraw, extra) {
   mrs_data <- mrs_data(data = data, ft = ft, resolution = res, ref = ref,
                        nuc = nuc, freq_domain = freq_domain, affine = NULL,
                        meta = meta, extra = extra)
-  return(mrs_data)
+  
+  if (verbose) {
+    cat(paste("N          :", N), "\n")
+    cat(paste("Npts       :", Npts), "\n")
+    cat(paste("Nframes    :", Nframes), "\n")
+    cat(paste("Prot. name :", prot_name), "\n")
+  }
+  
+  if (N == Npts) {
+    return(mrs_data)
+  } else if (N == 2 * Npts) {
+    metab_mrs <- crop_td_pts(mrs_data, end = Npts)
+    ref_mrs   <- crop_td_pts(mrs_data, start = Npts + 1)
+    out       <- list(metab = metab_mrs, ref = ref_mrs)
+    class(out) <- c("list", "mrs_data")
+    return(out)
+  } else {
+    warning(paste0("Expecting ", Npts, ", but found ", N, " data points."))
+    return(mrs_data)
+  }
 }
\ No newline at end of file
